@@ -108,8 +108,7 @@ struct Parse_Options_s
 struct Connector_set_s
 {
 	Connector ** hash_table;
-	int          table_size;
-	int          is_defined;  /* if 0 then there is no such set */
+	unsigned int table_size;
 };
 
 struct Dictionary_s
@@ -133,6 +132,11 @@ struct Dictionary_s
 	int u_strippable; /* units on left */
 	int s_strippable; /* generic suffix */
 	int p_strippable; /* generic prefix */
+	int r_stripped;
+	int l_stripped;
+	int u_stripped;
+	int s_stripped;
+	int p_stripped;
 	const char ** strip_left;
 	const char ** strip_right;
 	const char ** strip_units;
@@ -197,14 +201,14 @@ struct And_data_s
 
 struct Parse_info_struct
 {
-	int            x_table_size;
-	int            log2_x_table_size;
+	unsigned int   x_table_size;
+	unsigned int   log2_x_table_size;
 	X_table_connector ** x_table;
 	Parse_set *    parse_set;
 	int            N_words;
 	Disjunct **    chosen_disjuncts;
 	int            N_links;
-	Link          link_array[MAX_LINKS];
+	Link           link_array[MAX_LINKS];
 
 #ifdef USE_FAT_LINKAGES
 	/* Points to the image structure for each word.
@@ -240,7 +244,7 @@ struct Sentence_s
 	                               This is not the same as num alloced
 	                               because some may be non-canonical. */
 	int    num_valid_linkages;  /* number with no pp violations */
-	int    null_links;          /* null links allowed */
+	Boolean        null_links;  /* null links allowed */
 	int    null_count;          /* number of null links in linkages */
 	Parse_info     parse_info;  /* set of parses for the sentence */
 	Linkage_info * link_info;   /* array of valid and invalid linkages (sorted) */
