@@ -139,14 +139,6 @@ JNIEXPORT void JNICALL Java_org_linkgrammar_LinkGrammar_makeLinkage
 JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageNumViolations
 	(JNIEnv *, jclass);
 
-/*
- * Class:     LinkGrammar
- * Method:    linkageAndCost
- * Signature: ()I
- */
-JNIEXPORT jint JNICALL Java_org_linkgrammar_LinkGrammar_getLinkageAndCost
-	(JNIEnv *, jclass);
-
 /*
  * Class:     LinkGrammar
  * Method:    linkageDisjunctCost
@@ -227,23 +219,6 @@ JNIEXPORT jstring JNICALL Java_org_linkgrammar_LinkGrammar_getConstituentString
 JNIEXPORT jstring JNICALL Java_org_linkgrammar_LinkGrammar_getLinkString
 	(JNIEnv *, jclass);
 
-/*
- * Class:     LinkGrammar
- * Method:    isPastTenseForm
- * Signature: (Ljava/lang/String;)Z
- * @deprecated -- past-tense verbs are tagged with .v-d or .w-d or .q-d
- * subscripts. use those instead to figure out if a verb is past tense.
- */
-JNIEXPORT jboolean JNICALL Java_org_linkgrammar_LinkGrammar_isPastTenseForm
-	(JNIEnv *, jclass, jstring);
-
-/*
- * Class:     LinkGrammar
- * Method:    isEntity
- * Signature: (Ljava/lang/String;)Z
- */
-JNIEXPORT jboolean JNICALL Java_org_linkgrammar_LinkGrammar_isEntity
-	(JNIEnv *, jclass, jstring);
 
 #ifdef __cplusplus
 }
