@@ -187,7 +187,52 @@ strndup (const char *str, size_t size)
 /* ============================================================= */
 /* UTF8 utilities */
 
-int wctomb_check(char *s, wchar_t wc)
+#if defined(_MSC_VER) || defined(__MINGW32__)
+
+/** Returns length of UTF8 character.
+ * Current algo is based on the first character ony.
+ * If pointer is not pointing at first char, no not a valid value, returns 0.
+ * Returns 0 for NULL as well.
+ */
+static int get_utf8_charlen(const char *xc)
+{
+	unsigned char c;
+
+	c = (unsigned char) *xc;
+
+	if (c == 0) return 0;
+	if (c < 0x80) return 1;
+	if ((c >= 0xc2) && (c < 0xe0)) return 2; /* First byte of a code point U +0080 - U +07FF */
+	if ((c >= 0xe0) && (c < 0xf0)) return 3; /* First byte of a code point U +0800 - U +FFFF */
+	if ((c >= 0xf0) && (c <= 0xf4)) return 4; /* First byte of a code point U +10000 - U +10FFFF */
+	return -1; /* Fallthrough -- not the first byte of a code-point. */
+}
+
+/**
+ * (Experimental) Implementation of mbrtowc for Windows.
+ * This is required because the other, commonly available implementations
+ * seem to not work very well, based on user reports.  Someone who is
+ * really, really good at windows programming needs to review this stuff!
+ */
+size_t lg_mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
+{
+	int nb, nb2;
+
+	if (NULL == s) return 0;
+	if (0 == n) return -2;
+	if (0 == *s) { *pwc = 0; return 0; }
+
+	nb = get_utf8_charlen(s);
+	if (0 == nb) return 0;
+	if (0 > nb) return nb;
+	nb2 = MultiByteToWideChar(CP_UTF8, 0, s, nb, NULL, 0);
+	nb2 = MultiByteToWideChar(CP_UTF8, 0, s, nb, pwc, nb2);
+	if (0 == nb2) return 0;
+	return nb;
+}
+#endif /* defined(_MSC_VER) || defined(__MINGW32__) */
+
+static int wctomb_check(char *s, wchar_t wc)
 {
 	int nr;
 #ifdef _MSC_VER
@@ -197,6 +242,8 @@ int wctomb_check(char *s, wchar_t wc)
 		wprintf(L"Fatal Error: wctomb_check failed: %d %S\n", nr, &wc);
 		exit(1);
 	}
+	/* XXX TODO Perhaps the below needs to be uncommented?  .. tucker says no ... */
+	/* nr = nr / sizeof(wchar_t); */
 #else
 	mbstate_t mbss;
 	memset(&mbss, 0, sizeof(mbss));
@@ -211,19 +258,25 @@ int wctomb_check(char *s, wchar_t wc)
 
 /**
  * Downcase the first letter of the word.
+ * XXX FIXME This works 'most of the time', but is not techically correct.
+ * This is because towlower() and towupper() are locale dependant, and also
+ * because the byte-counts might not match up, e.g. German ß and SS.
+ * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
 void downcase_utf8_str(char *to, const char * from, size_t usize)
 {
 	wchar_t c;
 	int i, nbl, nbh;
 	char low[MB_LEN_MAX];
+	mbstate_t mbs;
 
-#ifdef _MSC_VER
-	nbh = MultiByteToWideChar(CP_UTF8, 0, from, 1, NULL, 0);
-	nbh = sizeof(wchar_t) * MultiByteToWideChar(CP_UTF8, 0, from, 1, &c, nbh);
-#else
-	nbh = mbtowc (&c, from, MB_CUR_MAX);
-#endif
+	memset(&mbs, 0, sizeof(mbs));
+	nbh = mbrtowc (&c, from, MB_CUR_MAX, &mbs);
+	if (nbh < 0)
+	{
+		prt_error("Error: Invalid multi-byte string!");
+		return;
+	}
 	c = towlower(c);
 	nbl = wctomb_check(low, c);
 
@@ -247,19 +300,25 @@ void downcase_utf8_str(char *to, const char * from, size_t usize)
 
 /**
  * Upcase the first letter of the word.
+ * XXX FIXME This works 'most of the time', but is not techically correct.
+ * This is because towlower() and towupper() are locale dependant, and also
+ * because the byte-counts might not match up, e.g. German ß and SS.
+ * The correct long-term fix is to use ICU or glib g_utf8_strup(), etc.
  */
 void upcase_utf8_str(char *to, const char * from, size_t usize)
 {
 	wchar_t c;
 	int i, nbl, nbh;
 	char low[MB_LEN_MAX];
+	mbstate_t mbs;
 
-#ifdef _MSC_VER
-	nbh = MultiByteToWideChar(CP_UTF8, 0, from, 1, NULL, 0);
-	nbh = sizeof(wchar_t) * MultiByteToWideChar(CP_UTF8, 0, from, 1, &c, nbh);
-#else
-	nbh = mbtowc (&c, from, MB_CUR_MAX);
-#endif
+	memset(&mbs, 0, sizeof(mbs));
+	nbh = mbrtowc (&c, from, MB_CUR_MAX, &mbs);
+	if (nbh < 0)
+	{
+		prt_error("Error: Invalid multi-byte string!");
+		return;
+	}
 	c = towupper(c);
 	nbl = wctomb_check(low, c);
 
@@ -471,14 +530,22 @@ void exfree(void * p, size_t size)
 char * join_path(const char * prefix, const char * suffix)
 {
 	char * path;
-	int path_len;
+	size_t path_len, prel;
 
 	path_len = strlen(prefix) + 1 /* len(DIR_SEPARATOR) */ + strlen(suffix);
 	path = (char *) malloc(path_len + 1);
 
 	strcpy(path, prefix);
-	path[strlen(path)+1] = '\0';
-	path[strlen(path)] = DIR_SEPARATOR;
+
+	/* Windows is allergic to multiple path separators, so append one
+	 * only if the prefix isn't already terminated by a path sep.
+	 */
+	prel = strlen(path);
+	if (0 < prel && path[prel-1] != DIR_SEPARATOR)
+	{
+		path[prel] = DIR_SEPARATOR;
+		path[prel+1] = '\0';
+	}
 	strcat(path, suffix);
 
 	return path;
@@ -659,7 +726,7 @@ char * dictionary_get_data_dir(void)
 				printf("   FAIL GetModuleFileName for current program \n");
 			}
 		}
-   }  /* BHayes - end added else block */
+   }
 #endif
 
 	return data_dir;
@@ -707,8 +774,15 @@ void * object_open(const char *filename,
 	/* Look for absolute filename.
 	 * Unix: starts with leading slash.
 	 * Windows: starts with C:\  except that the drive letter may differ.
+	 * Note that only native windows C library uses backslashs; mingw 
+	 * seems to use forward-slash, from what I can tell.
 	 */
-	if ((filename[0] == '/') || ((filename[1] == ':') && (filename[2] == '\\')))
+	if ((filename[0] == '/')
+#if defined(_WIN32) || defined(_MSC_VER) || defined(__MINGW32__)
+		|| ((filename[1] == ':')
+			 && ((filename[2] == '\\') || (filename[2] == '/')))
+#endif
+	)
 	{
 		/* fopen returns NULL if the file does not exist. */
 		fp = opencb(filename, user_data);
@@ -789,7 +863,7 @@ FILE *dictopen(const char *filename, const char *how)
 	 * in the *same* directory in which the first one was found.
 	 * (The first one is typcailly "en/4.0.dict")
 	 * The global "path_found" records where the first dict was found.
-	 * The goal here is to avoid fractured install insanity.
+	 * The goal here is to avoid insanity due to user's fractured installs.
 	 */
 	if (path_found)
 	{
