@@ -88,6 +88,16 @@ char * strndup (const char *str, size_t size);
  * Suck it up!
  */
 #define rand_r(seedp) rand()
+
+/* Users report that the default mbrtowc that comes with windows and/or
+ * cygwin just doesn't work very well. So we use our own custom version,
+ * instead.
+ */
+size_t lg_mbrtowc(wchar_t *, const char *, size_t, mbstate_t *);
+#ifdef mbrtowc
+#undef mbrtowc
+#endif
+#define mbrtowc(w,s,n,x) lg_mbrtowc(w,s,n,x)
 #endif /* _MSC_VER || __MINGW32__ */
 
 /*
@@ -178,7 +188,7 @@ static inline size_t utf8_strlen(const char *s)
  */
 static inline size_t utf8_next(const char *s)
 {
-	int n = 0;
+	size_t n = 0;
 	while (0 != *s)
 	{
 		if ((0x80 <= ((unsigned char) *s)) && 
@@ -196,6 +206,7 @@ static inline int is_utf8_upper(const char *s)
 
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+	if (nbytes < 0) return 0;  /* invalid mb sequence */
 	if (iswupper(c)) return nbytes;
 	return 0;
 }
@@ -208,6 +219,7 @@ static inline int is_utf8_alpha(const char *s)
 
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+	if (nbytes < 0) return 0;  /* invalid mb sequence */
 	if (iswalpha(c)) return nbytes;
 	return 0;
 }
@@ -220,6 +232,7 @@ static inline int is_utf8_digit(const char *s)
 
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+	if (nbytes < 0) return 0;  /* invalid mb sequence */
 	if (iswdigit(c)) return nbytes;
 	return 0;
 }
@@ -232,6 +245,7 @@ static inline int is_utf8_space(const char *s)
 
 	memset(&mbs, 0, sizeof(mbs));
 	nbytes = mbrtowc(&c, s, MB_CUR_MAX, &mbs);
+	if (nbytes < 0) return 0;  /* invalid mb sequence */
 	if (iswspace(c)) return nbytes;
 	return 0;
 }
@@ -263,6 +277,7 @@ static inline int utf8_upper_match(const char * s, const char * t)
 
 	ns = mbrtowc(&ws, s, MB_CUR_MAX, &mbs);
 	nt = mbrtowc(&wt, t, MB_CUR_MAX, &mbt);
+	if (ns < 0 || nt < 0) return FALSE;  /* invalid mb sequence */
 	while (iswupper(ws) || iswupper(wt))
 	{
 		if (ws != wt) return FALSE;
@@ -270,11 +285,11 @@ static inline int utf8_upper_match(const char * s, const char * t)
 		t += nt;
 		ns = mbrtowc(&ws, s, MB_CUR_MAX, &mbs);
 		nt = mbrtowc(&wt, t, MB_CUR_MAX, &mbt);
+		if (ns < 0 || nt < 0) return FALSE;  /* invalid mb sequence */
 	}
 	return TRUE;
 }
 
-int wctomb_check(char *s, wchar_t wc);
 void downcase_utf8_str(char *to, const char * from, size_t usize);
 void upcase_utf8_str(char *to, const char * from, size_t usize);
 
